/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"
#include "FolderItem.h"
#include "PopupMenu.h"
#include "TitleItem.h"
#include "Painter.h"
int FolderItem::m_nOverlapX = 5;
int FolderItem::m_nOverlapY = 5;
int FolderItem::m_nDelay = 0;
BOOL FolderItem::m_bHoverOpen = TRUE;

FolderItem::FolderItem(PopupMenu* pSubMenu, LPCSTR pszTitle) : TitleItem(pszTitle)
{
	m_pSubMenu = pSubMenu;
	m_uTimerId = 0;
	m_nSortPriority = 10;
}

FolderItem::~FolderItem()
{
	if (m_pSubMenu)
		delete m_pSubMenu;
	m_pSubMenu = NULL;
}

// Paints the folder item, the parent class will paint first
// then will we add the |> icon
void FolderItem::Paint(HDC hDC)
{
	// now TitleItem handles the arrow too (based on IsLeaf)
	TitleItem::Paint(hDC);
}

void FolderItem::GetTitleRect(RECT* r)
{
	TitleItem::GetTitleRect(r);
	// r->right = r->right - (m_bDrawArrow ? 16 : 0);
}


void FolderItem::Attached(PopupMenu* pParentMenu)
{
	TitleItem::Attached(pParentMenu);

	m_pParent->AddChild(m_pSubMenu);
	m_pSubMenu->SetParent(m_pParent);
}


void FolderItem::Invoke()
{
	if(!m_bHoverOpen)
    {
		ShowSubMenu();
	}

	Active(TRUE);
}

BOOL FolderItem::Active(BOOL bActivate)
{
	BOOL result = TitleItem::Active(bActivate);

	if (bActivate)
	{
		if(m_uTimerId == 0 && m_bHoverOpen)
		{
			m_uTimerId = (UINT_PTR)this;
			SetTimer(GetWindow(), m_uTimerId, m_nDelay, NULL);
		}
	}
	else
	{
		if (m_uTimerId != 0)
        {
			KillTimer(GetWindow(), m_uTimerId);
        }

		m_uTimerId = 0;

		if (m_pSubMenu != NULL)
		{
			if (IsWindowVisible(m_pSubMenu->GetWindow()))
            {
				m_pSubMenu->Hide(HIDE_CHILDREN);
            }
		}
	}

	return result;
}

void FolderItem::Timer(UINT_PTR uTimer)
{
	if (uTimer == m_uTimerId)
	{
		ShowSubMenu();
		KillTimer(GetWindow(), m_uTimerId);
		m_uTimerId = 0;
	}
}

void FolderItem::ShowSubMenu()
{
    RECT r;
    RECT s;
    RECT rParent;
    PopupMenu* pParent;
    int x, y;

    HMONITOR hMon = LSMonitorFromWindow(GetWindow(),MONITOR_DEFAULTTONEAREST);
    MONITORINFO mi;
    mi.cbSize = sizeof(mi);
    LSGetMonitorInfo(hMon, &mi);

    // This window
    GetWindowRect(GetWindow(), &r);

    // Sub folder window
    GetWindowRect(m_pSubMenu->GetWindow(), &s);

    pParent = m_pParent->GetParent();
    if (pParent)
        GetWindowRect(pParent->GetWindow(), &rParent);

    /* push up if it'd go past the bottom */ 
    y = r.top + m_nTop + m_nOverlapY;
    if (y + (s.bottom - s.top) >= mi.rcMonitor.bottom)
        y = mi.rcMonitor.bottom - (s.bottom - s.top);

    /* but ya know, it's annoying if the top's on a dif. monitor */ 
    if (y < mi.rcMonitor.top)
        y = mi.rcMonitor.top;

    if (pParent && (rParent.left > r.left))
        // if my window is to the left of my parent menu
        x = r.left - (s.right - s.left) + m_nOverlapX;
    else
        // else try to go right
        x = r.right - m_nOverlapX;

    /* flip over if it'd go past the right edge */ 
    if (x + (s.right - s.left) >= mi.rcMonitor.right)
        x = r.left - (s.right - s.left) + m_nOverlapX;

    // if the new window is too far to the left
    if (x < mi.rcMonitor.left)
        x = r.right - m_nOverlapX;

    // keep focus on this folder..
    SetForegroundWindow(GetWindow());

    m_pSubMenu->Show(x, y);
}

BOOL FolderItem::Key(int nKey)
{
	BOOL bConsumed = FALSE;
	if (IsActive())
	{
		switch (nKey)
		{
			
            case VK_RIGHT:
			ShowSubMenu();

			// move the focus to the folder
			SetForegroundWindow(m_pSubMenu->GetWindow());

			// send a vk_down, a hack to make the user not having to press
			// arrow down to ...
			PostMessage(m_pSubMenu->GetWindow(), WM_KEYDOWN, VK_DOWN, 0);
			bConsumed = TRUE;
			break;
			case VK_LEFT:
			if (m_pParent->GetParent() != NULL)
			{
				m_pSubMenu->Hide(HIDE_CHILDREN);
				SetForegroundWindow(GetWindow());
				bConsumed = TRUE;
			}
			break;
		}
	}

	if (m_nShortcut == nKey)
	{
		// HACK to activate this menuitem and disable the
		// other menuitems.
		m_pParent->Mouse(WM_MOUSEMOVE, 0, m_nTop);
		bConsumed = TRUE;
	}
	return bConsumed;
}
