/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"
#include "commandItem.h"
#include "info.h"
#include "TitleItem.h"
#include "PopupMenu.h"
#include "contextmenu.h"

CommandItem::CommandItem(LPCSTR pszCommand, LPCSTR pszArgument, LPCSTR pszTitle) :
		TitleItem(pszTitle)
{
    m_pszCommand = pszCommand != NULL ? _strdup(pszCommand) : NULL;
	m_pszArgument = pszArgument != NULL ? _strdup(pszArgument) : NULL;
}

CommandItem::~CommandItem()
{
	if (m_pszCommand)
		free(m_pszCommand);
	m_pszCommand = NULL;
	if (m_pszArgument)
		free(m_pszArgument);
	m_pszArgument = NULL;
}


void CommandItem::Invoke()
{
	m_pParent->Hide(HIDE_PARENTS);

    // Call parent
    MenuItem::Invoke();

    LSExecuteEx(NULL, NULL,
        m_pszCommand, m_pszArgument, NULL, SW_SHOWNORMAL);
}

UINT CommandItem::GetDrawTextFormat()
{
	if (m_pszCommand != NULL)
	{
		// if it is not a bang command and the command string
		// contains &, then disable prefix.
        
        // *Popup "Favorites" !DynamicFolder:"$Favorites$"
        // didn't show "Dark & Shattered Lands" right - it was always aligned to the left
        // hence the GetAlignment() was added, dunno if it was left out on purpose.
        // if it was please leave a comment with the reason(s) in here :)
		if (m_pszCommand[0] != '!' && strchr(m_pszCommand, '&'))
			return GetAlignment() | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS | DT_NOPREFIX | DT_NOCLIP;
	}

	return TitleItem::GetDrawTextFormat();
}

void CommandItem::RInvoke()
{
	PopupContextMenu(m_pszCommand);
}
